import 'package:flutter/material.dart';

/// Tipos de animaciones
enum AnimationType { fadeIn, normal }

/// Main class, [context] es el BuildContext de la aplicacion en ese momento
/// [child] es la pagina a la que se desea navegar
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions({
    required this.context,
    required this.child,
    this.animation = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
    this.replacement = false,
  }) {
    switch (animation) {
      case AnimationType.normal:
        _normalTransition();
        break;
      case AnimationType.fadeIn:
        _fadeInTransition();
        break;
    }
  }

  /// handler de la transicion normal
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => child);
    replacement ? _pushReplacementPage(route) : _pushPage(route);
  }

  /// handler de la transicion con fade in
  void _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: Curves.easeOut),
            ),
          );
        });

    replacement ? _pushReplacementPage(route) : _pushPage(route);
  }

  /// push de la pagina normal
  void _pushPage(Route route) => Navigator.push(context, route);

  /// push de la pagina reemplazando la anterior
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);
}
